demo_application = '''from giotto.programs import GiottoProgram, ProgramManifest
from giotto.views import BasicView

class ColoredMultiplyView(BasicView):

    def text_html(self, result):
        return """<!DOCTYPE html>
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.8/jquery.min.js"></script>
        <html>
            <body>
                <span style="color: blue">%(x)s * %(y)s</span> == 
                <span style="color: red">%(product)s</span>
            </body>
        </html>""" % result

    def text_cmd(self, result):
        from colorama import init, Fore
        init()
        return "{blue}{x} * {y}{reset} == {red}{product}{reset}".format(
            blue=Fore.BLUE,
            red=Fore.RED,
            reset=Fore.RESET,
            x=result['x'],
            y=result['y'],
            product=result['product'],
        )

    def text_irc(self, result):
        return "{blue}{x} * {y}{reset} == {red}{product}{reset}".format(
            blue="\x0302",
            red="\x0304",
            reset="\x03",
            x=result['x'],
            y=result['y'],
            product=result['product'],
        )


def multiply(x, y):
    return {'x': int(x), 'y': int(y), 'product': int(x) * int(y)}


manifest = ProgramManifest({
    'multiply': GiottoProgram(
        controllers = ('http-get', 'cmd', 'irc'),
        model=[multiply, {'x': 3, 'y': 3, 'product': 9}],
        view=ColoredMultiplyView
    )
})'''


import inspect
import re
import os

from giotto.exceptions import ProgramNotFound, MockNotFound, ControlMiddlewareInterrupt
from giotto.utils import super_accept_to_mimetype
from giotto.control import GiottoControl
from giotto.views import GiottoView

class GiottoProgram(object):
    name = None
    pre_input_middleware = ()
    input_middleware = ()
    controllers = ()
    cache = 0
    model = ()
    view = None
    output_middleware = ()

    def __init__(self, **kwargs):
        self.__dict__ = kwargs
        if hasattr(self.view, 'mro') and GiottoView in self.view.mro():
            self.view = self.view()

    def get_model_args_kwargs(self):
        """
        Inspect the model (or view in the case of no model) and return the args
        and kwargs. This functin is necessary because argspec returns in a silly format
        by default.
        """
        source = self.get_model()
        if not source:
            return [], {}

        if hasattr(source, 'render'):
            # if 'source' is a view object, try to get the render method,
            # otherwise, just use the __call__ method.
            source = source.render

        argspec = inspect.getargspec(source)
        kwargs = dict(zip(*[reversed(l) for l in (argspec.args, argspec.defaults or [])]))
        args = [x for x in argspec.args if x not in kwargs.keys()]
        if args and args[0] == 'cls':
            args = args[1:]
        return args, kwargs

    def get_model(self):
        if len(self.model) == 0:
            return None
        return self.model[0]

    def get_model_mock(self):
        if not self.model or not self.model[0]:
            # no mock needed
            return {}
        try:
            return self.model[1]
        except IndexError:
            raise MockNotFound("no mock for %s" % self.name)

    def execute_input_middleware_stream(self, request, controller):
        """
        Request comes from the controller. Returned is a request.
        controller arg is the name of the controller.
        """
        controller = "".join(controller.split('-')[:1])
        middlewares = list(self.pre_input_middleware) + list(self.input_middleware)
        for m in middlewares:
            to_execute = getattr(m(), controller)
            if to_execute:
                result = to_execute(request)
                if GiottoControl in type(result).mro():
                    # a middleware class returned a control object (redirection, et al.)
                    raise ControlMiddlewareInterrupt(control=result)
                request = result
        return request

    def execute_output_middleware_stream(self, request, response, controller):
        controller = "".join(controller.split('-')[:1]) # 'http-get' -> 'http'
        for m in self.output_middleware:
            to_execute = getattr(m(), controller, None)
            if to_execute:
                response = to_execute(request, response)
        return response

    def execute_model(self, data):
        """
        Returns data from the model, if mock is defined, it returns that instead.
        """
        model = self.get_model()
        if model is None:
            return None
        return model(**data)

    def execute_view(self, data, mimetype, errors):
        if not self.view:
            return {'body': '', 'mimetype': ''}
        return self.view.render(data, mimetype, errors)

class ProgramManifest(object):
    """
    Represents a node in a larger manifest tree. Manifests are like URLS for
    giotto applications. All keys must be strings, and all values must be
    either GiottoPrograms or another ProgramManifest instance.
    """
    key_regex = r'^\w*$'

    def __init__(self, manifest, backname=None):
        self.manifest = manifest
        # any sub manifests, convert to manifests objects
        for key, item in self.manifest.items():
            type_ = type(item)

            is_program = isinstance(item, GiottoProgram)
            is_manifest = type_ == ProgramManifest
            is_list = type_ == list

            if not re.match(self.key_regex, key):
                raise ValueError("Invalid manifest key: %s" % key)

            if type_ is dict:
                self.manifest[key] = ProgramManifest(item, backname=key)
            elif not is_manifest and not is_program and not is_list:
                msg = "Manifest value must be either: a program, a list of programs, or another manifest"
                raise TypeError(msg)

    def __repr__(self):
        return "<Manifest %s (%s nodes)>" % (self.backname, len(self.manifest))

    def __getitem__(self, key):
        return self.manifest[key]

    def get_program(self, program_name, controller_tag):
        """
        Find the program within this manifest. If key is found, and it contains
        a list, iterate over the list and return the program that matches
        the controller tag. 
        """
        result = self.manifest[program_name]
        if type(result) == ProgramManifest:
            return result
        if type(result) is not list:
            result = [result]
        for program in result:
            if controller_tag in program.controllers:
                return program

        # we looped through all programs and found no match, maybe one program
        # has no explicitly defined controller tag? If so return that.
        for program in result:
            if not program.controllers or '*' in program.controllers:
                return program

        # we found the key, and looped through all programs, but the controller
        # tag could not be found.
        msg = "Program '%s' does not allow '%s' controller" % (program_name, controller_tag)
        raise ProgramNotFound(msg)

    def extract_superformat(self, name):
        """
        In comes the program name, out comes the superformat (html, json, xml, etc)
        and the new program name with superstring removed.
        """
        if '.' in name:
            splitted = name.split('.')
            return (splitted[0], splitted[1])
        else:
            return (name, None)

    def parse_invocation(self, invocation, controller_tag):
        if invocation.endswith('/'):
            invocation = invocation[:-1]
        if invocation.startswith('/'):
            invocation = invocation[1:]

        splitted_path = invocation.split('/')
        start_name = splitted_path[0]
        start_args = splitted_path[1:]

        parsed = self._parse(start_name, start_args, controller_tag)
        parsed['invocation'] = invocation
        return parsed

    def _parse(self, raw_program_name, args, controller_tag):
        """
        Recursive function to transversing nested manifests.
        raw_program_name == program with superformat intact
        """
        program_name, superformat = self.extract_superformat(raw_program_name)
        try:
            program = self.get_program(program_name, controller_tag)
        except KeyError:
            # program name is not in keys, drop down to root...
            if '' in self.manifest:
                result = self.get_program('', controller_tag)
                if type(result) == ProgramManifest:
                    return result._parse(raw_program_name, args, controller_tag)
                else:
                    return {
                        'program': result,
                        'name': '',
                        'superformat': superformat,
                        'superformat_mime': None,
                        'args': [program_name] + args,
                    }
            else:
                raise ProgramNotFound("Program '%s' Does Not Exist" % program_name)
        else:
            if type(program) == ProgramManifest:
                if program_name == '':
                    return program._parse('', args, controller_tag)
                if not args:
                    raise ProgramNotFound('No root program for namespace, and no program match')
                return program._parse(args[0], args[1:], controller_tag)
            else:
                return {
                    'program': program,
                    'name': program_name,
                    'superformat': superformat,
                    'superformat_mime': super_accept_to_mimetype(superformat),
                    'args': args,
                }

from giotto.programs.shell import Shell
from giotto.programs.make_tables import MakeTables
management_manifest = ProgramManifest({
    'make_tables': MakeTables(),
    'shell': Shell(),
})


import os

from giotto.utils import parse_kwargs
from giotto.controllers import GiottoController
from giotto.control import Redirection

cmd_execution_snippet = """
args = sys.argv
mock = '--model-mock' in args
if mock:
    # remove the mock argument so the controller doesn't get confused
    args.pop(args.index('--model-mock'))
from giotto.controllers.cmd import CMDController, CMDRequest
request = CMDRequest(sys.argv)
controller = CMDController(request=request, manifest=manifest, model_mock=mock)
controller.get_response()"""


def make_cmd_invocation(invocation, args, kwargs):
    """
    >>> make_cmd_invocation('path/program', ['arg1', 'arg2'], {'darg': 4})
    ['./giotto-cmd', '/path/program/arg1/arg2/', '--darg=4']
    """
    if not invocation.endswith('/'):
        invocation += '/'
    if not invocation.startswith('/'):
        invocation = '/' + invocation

    cmd = invocation

    for arg in args:
        cmd += str(arg) + "/"

    rendered_kwargs = []
    for k, v in kwargs.iteritems():
        rendered_kwargs.append("--%s=%s" % (k,v))
    
    return ['./giotto-cmd', cmd] + rendered_kwargs

class CMDRequest(object):
    def __init__(self, argv):
        self.enviornment = os.environ
        self.argv = argv

class CMDController(GiottoController):
    """
    Basic command line cntroller class. self.request will be the value found
    in sys.argv; a list of commandline arguments with the first item being the 
    name of the script ('giotto'), the second being the name of th program
    and the rest being commandline arguments.
    """
    name = 'cmd'
    default_mimetype = 'text/x-cmd'

    def get_invocation(self):
        return self.request.argv[1]

    def get_controller_name(self):
        return 'cmd'

    def get_raw_data(self):
        """
        Parse the raw commandline arguments (from sys.argv) to a dictionary
        that is understandable to the rest of the framework.
        """
        arguments = self.request.argv[1:]
        if not arguments[0].startswith('--'):
            # first argument is the program name
            arguments = arguments[1:]
        return parse_kwargs(arguments)

    def get_concrete_response(self):
        result = self.get_data_response()
        
        if type(result) == Redirection:
            invocation, args, kwargs = result.rendered_invocation
            rendered_invocation = make_cmd_invocation(invocation, args, kwargs)
            req = CMDRequest(rendered_invocation)
            return CMDController(req, self.manifest, self.model_mock)
        else:
            response = {
                'stdout': [result['body']],
                'stderr': [],
            }

        # now do middleware
        stdout = response['stdout']

        if hasattr(stdout, 'write'):
            # returned is a file, print out the contents through stdout
            print(stdout.write())
        else:
            for line in stdout:
                print(line)

        for line in response['stderr']:
            sys.stderr.write(line)


    def get_primitive(self, name):
        return None # implement later



from .models import User
from giotto.exceptions import NotAuthorized
from giotto.control import Redirection
from giotto import config
from giotto.utils import random_string

class AuthenticationMiddleware(object):
    """
    This input middleware class must preceed any other authentication middleware class.
    It is used to extract authentiction information from the request, and
    verify that the credientials are correct.
    """
    def http(self, request):
        user = None
        if request.method == 'POST':
            u = request.form.get('username', None)
            p = request.form.get('password', None)
            user = User.get_user_by_password(u, p) if u and p else None

        session_key = request.cookies.get('giotto_session', None)
        if session_key:
            username = config.auth_session.get(session_key)
            user = config.session.query(User).filter_by(username=username).first()

        setattr(request, 'user', user)
        return request

    def cmd(self, request):
        return request


class SetAuthenticationCookies(object):
    """
    Place this middleware class in the output stream to set the cookies that
    are used to authenticate each subsequent request.
    """

    def make_session(self, user):
        """
        Create a session for the user, and then return the key.
        """
        session_key = random_string(15)
        while config.auth_session.get(session_key):
            session_key = random_string(15)
        config.auth_session.set(session_key, user.username, config.auth_session_expire)
        return session_key

    def http(self, request, response):
        if hasattr(request, 'user') and request.user:
            username = request.user.username
            password = request.user.password
            user = User.get_user_by_hash(username, password)
        elif 'password' in request.form and 'password' in request.form:
            # user just registered.
            username = request.form['username']
            password = request.form['password']
            user = User.get_user_by_password(username, password)
        
        if not user:
            return response

        session_key = self.make_session(user)
        response.set_cookie('giotto_session', session_key)

        return response

    def cmd(self, request, response):
        return response


class AuthenticatedOrDie(object):
    """
    Put this in the input middleware stream to fail any requests that aren't
    made by authenticated users
    """
    def http(self, request):
        if not request.user:
            raise NotAuthorized('Must be Logged in for this program')
        return request


def AuthenticatedOrRedirect(invocation, args=[], kwargs={}):
    """
    Middleware class factory that redirects if the user is not logged in.
    Otherwise, nothing is effected.
    """
    class AuthenticatedOrRedirect(object):
        def http(self, request):
            if request.user:
                return request
            return Redirection(invocation, args, kwargs)
    return AuthenticatedOrRedirect


def NotAuthenticatedOrRedirect(invocation, args=[], kwargs={}):
    """
    Middleware class factory that redirects if the user is not logged in.
    Otherwise, nothing is effected.
    """
    class NotAuthenticatedOrRedirect(object):
        def http(self, request):
            if not request.user:
                return request
            return Redirection(invocation, args, kwargs)
    return NotAuthenticatedOrRedirect


class LogoutMiddleware(object):
    def http(self, request, response):
        response.delete_cookie('giotto_session')
        return response


from collections import defaultdict
from giotto.utils import Mock

class InvalidInput(Exception):
    def __init__(self, message=None, data=None):
        if not data:
            self.__dict__ = defaultdict(lambda: '', {'message': message})
        else:
            self.__dict__ = defaultdict(lambda: '', data)

    def __getitem__(self, item):
        # be permissive of attribute errors because jinja templates
        # need to not blow up when there are no errors.
        if not item in self.__dict__:
            return Mock()

class InvalidProgram(Exception):
    pass

class ProgramNotFound(Exception):
    pass

class DataNotFound(Exception):
    pass

class NoViewMethod(Exception):
    pass

class MockNotFound(Exception):
    pass

class NotAuthorized(Exception):
    pass

class ControlMiddlewareInterrupt(Exception):
    def __init__(self, message=None, control=None):
        self.control = control
        self.message = message

        # render the control object now because this object will never
        self.control(None).render('some_mimetype')


import copy
import urllib

from giotto.exceptions import NoViewMethod, InvalidInput
from giotto.controllers import GiottoController
from giotto.control import Redirection
from werkzeug.wrappers import Request, Response
from werkzeug.utils import redirect


http_execution_snippet = """
mock = '--model-mock' in sys.argv
from werkzeug.serving import run_simple
from giotto.controllers.http import make_app
application = make_app(manifest, model_mock=mock)
if '--run' in sys.argv:
    run_simple('127.0.0.1', 5000, application, use_debugger=True, use_reloader=True)"""


def make_url(invocation, args=[], kwargs={}):
    """
    >>> make_url('some/path/program', ['arg1', 'arg2'], {'arg3': 4})
    '/some/path/program/arg1/arg2?arg3=4'
    """
    if not invocation.endswith('/'):
        invocation += '/'
    if not invocation.startswith('/'):
        invocation = '/' + invocation

    url = invocation

    for arg in args:
        url += str(arg) + "/"

    if kwargs:
        url = url[:-1]
        url += "?" + urllib.urlencode(kwargs)
    
    return url


class HTTPController(GiottoController):
    name = 'http'
    default_mimetype = 'text/html'

    def mimetype_override(self):
        accept = self.request.headers['Accept']
        has_json_in_view = any([x for x in self.program.view.render_map.keys() if 'json' in x])
        if accept == '*/*' and self.request.is_xhr and has_json_in_view:
            # return json on ajax calls if no accept headers are present.
            # and only if the view has implemented a application/json method
            return "application/json"

        if accept != '*/*' and not accept.startswith('text/html'):
            return accept
        return None

    def get_invocation(self):
        return self.request.path

    def get_controller_name(self):
        return 'http-%s' % self.request.method.lower()

    def get_raw_data(self):
        data = {}
        if self.request.method == 'GET':
            data = self.request.args
        elif self.request.method == 'POST':
            data = self.request.form
        return data

    def get_concrete_response(self):
        code = 200
        try:
            result = self.get_data_response()
        except NoViewMethod:
            code = 415
            result = {
                'mimetype': 'text/plain',
                'body': 'Unsupported Media Type: %s' % self.mimetype
            }
        except InvalidInput as exc:
            request = make_duplicate_request(self.request)
            request.method = 'GET'
            c = HTTPController(request, self.manifest, self.model_mock, errors=exc)
            response = c.get_response()
            response.status_code = 400
            return response

        if type(result) == Redirection:
            invocation, args, kwargs = result.rendered_invocation
            response = redirect(make_url(invocation, args, kwargs))
        else:
            response = Response(
                status=code,
                response=result['body'],
                mimetype=result['mimetype'],
            )

        return response

    def get_primitive(self, primitive):
        if primitive == 'RAW_PAYLOAD':
            return self.get_raw_data()
        if primitive == 'LOGGED_IN_USER':
            return self.request.user
        if primitive == 'ALL_PROGRAMS':
            return self.manifest.get_all_programs()


def make_app(manifest, model_mock=False, cache=None):
    
    def application(environ, start_response):
        """
        WSGI app for serving giotto applications
        """
        request = Request(environ)
        controller = HTTPController(request, manifest, model_mock=model_mock)
        wsgi_response = controller.get_response()
        return wsgi_response(environ, start_response)

    return application


def make_duplicate_request(request):
    """
    Since werkzeug request objects are immutable, this is needed to create an
    identical reuet object with immutable values so it can be retried after a
    POST failure.
    """
    class FakeRequest(object):
        method = 'GET'
        path = request.path
        headers = request.headers
        args = request.args
        user = request.user
        cookies = request.cookies
        is_xhr = request.is_xhr
    return FakeRequest()


from giotto.views import GiottoView

class BasicRegisterForm(GiottoView):
    def text_html(self, result):
        return """<!DOCTYPE html>
        <html>
            <body>
                <form method="post" action="">
                    username: <input name="username" type="text"><br>
                    password: <input name="password" type="password"><br>
                    <input type="submit">
                </form>
            </body>
        </html>
        """


class GiottoControl(object):
    pass

class Redirection(GiottoControl):
    """
    Represents the result of one invocation as the result of another.
    """
    rendered_invocation = ()
    render_map = {} # because regular giottoviews have this. This is always empty

    def __init__(self, invocation, args=[], kwargs={}):
        self.invocation = invocation
        self.args = args
        self.kwargs = kwargs

    def render_args_kwargs(self, model):
        new_args = []
        for arg in self.args:
            if type(arg) == _M:
                new_args.append(arg.get_value(model))
        
        new_kwargs = {}
        for k, v in self.kwargs.iteritems():
            if type(v) == _M:
                new_kwargs[k] = v.get_value(model)

        self.rendered_invocation = (self.invocation, new_args, new_kwargs)

    def render(self, result, mimetype, errors):
        self.render_args_kwargs(result)
        return self

class _M(object):
    """
    Represents the getting of a value on an abstract model retun value.
    For internal use only.
    """
    def __init__(self, attribute=None, brackets=None):
        self.attribute = attribute
        self.brackets = brackets

    def get_value(self, model):
        if self.brackets:
            return model[self.brackets]

        if self.attribute:
            return getattr(model, self.attribute)

class MClass(object):
    """
    Symbolic representation the return value of the model
    """
    def __getattr__(self, item):
        return _M(attribute=item)

    def __getitem__(self, item):
        return _M(brackets=item)

M = MClass()


from setuptools import setup

setup(
	name='giotto',
    version='0.9.6',
    description='MVC Application Framework',
    long_description=open('README.rst').read(),
    author='Chris Priest',
    author_email='cp368202@ohiou.edu',
    url='https://github.com/priestc/giotto',
    packages=[
        'giotto',
        'giotto.controllers',
        'giotto.programs',
        'giotto.contrib',
        'giotto.contrib.auth',
        'giotto.contrib.static',
        'giotto.views'
    ],
    scripts=['giotto/giotto_project'],
    license='LICENSE',
    install_requires=[
        'werkzeug',
        'irc',
        'jinja2',
        'py-bcrypt',
        'mimeparse',
        'python-magic',
        'sqlalchemy',
    ],
)


from giotto.programs import GiottoProgram
from giotto.views import BasicView

def make_tables():
    from giotto import config
    Base = config.Base
    engine = config.engine
    Base.metadata.create_all(engine)
    return 'All tables created'

class MakeTables(GiottoProgram):
    """
    Program for creating the database tables for all imported models. Use this
    program internaly only. Do not hook it up through HTTP.
    """
    controllers = ('cmd', )
    model = [make_tables]
    view = BasicView


import argparse
import string
import random
import json

from collections import defaultdict
from sqlalchemy.ext.declarative import declarative_base, declared_attr
from sqlalchemy.orm import class_mapper, ColumnProperty

import giotto

class Mock(object):
    """
    This class creates an object that will replace the error object when no error
    object exists. It allows arbirtary attribute getting without raising an
    attribute error.
    >>> m = Mock()
    >>> m.nothing.empty.made_up.wut.lol
    ''
    >>> bool(m.wut)
    False
    """
    def __getattribute__(self, item):
        return Mock()

    def __str__(self):
        return ''

    def __nonzero__(self):
        return False

def parse_kwargs(kwargs):
    """
    Convert a list of kwargs into a dictionary. Duplicates of the same keyword
    get added to an list within the dictionary.

    >>> parse_kwargs(['--var1=1', '--var2=2', '--var1=3']
    {'var1': [1, 3], 'var2': 2}
    """
    
    d = defaultdict(list)
    for k, v in ((k.lstrip('-'), v) for k,v in (a.split('=') for a in kwargs)):
        d[k].append(v)

    ret = {}
    for k, v in d.iteritems():
        # replace single item lists with just the item.
        if len(v) == 1 and type(v) is list:
            ret[k] = v[0]
        else:
            ret[k] = v
    return ret

def super_accept_to_mimetype(ext):
    if not ext:
        return
    if ext.startswith('.'):
        ext = ext[1:]
    if ext in ('jpeg', 'jpg'):
        return 'image/jpeg'
    if ext == 'gif':
        return 'image/gif'
    if ext == 'txt':
        return 'text/plain'
    if ext in ('html', 'htm'):
        return 'text/html'
    if ext == 'json':
        return 'application/json'
    if ext == 'css':
        return "text/css"


def initialize_giotto(config):
    setattr(giotto, 'config', config)

def random_string(n):
    return ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(n))

def better_base():
    """
    Return a better SQLAlchemy Base class, for making things a little easier to
    work with.
    """
    Base = declarative_base()
    class BetterBase(Base):
        __abstract__ = True

        @declared_attr
        def __tablename__(cls):
            return "giotto_" + cls.__name__.lower()

        @classmethod
        def attribute_names(cls):
            return [prop.key for prop in class_mapper(cls).iterate_properties
                if isinstance(prop, ColumnProperty)]

        def todict(self):
            attrs = self.attribute_names()
            return [(attr, getattr(self, attr)) for attr in attrs]

    return BetterBase

def htmlize(value):
    """
    Turn any object into a html viewable entity.
    """
    return str(value).replace('<', '&lt;').replace('>', '&gt;')

def htmlize_list(items):
    """
    Turn a python list into an html list.
    """
    out = ["<ul>"]
    for item in items:
        out.append("<li>" + htmlize(item) + "</li>")
    out.append("</ul>")
    return "\n".join(out)

def pre_process_json(obj):
    """
    Preprocess items in a dictionary or list and prepare them to be json serialized.
    """
    if type(obj) is dict:
        new_dict = {}
        for key, value in obj.iteritems():
            new_dict[key] = pre_process_json(value)
        return new_dict

    elif type(obj) is list:
        new_list = []
        for item in list:
            new_list.append(pre_process_json(item))
        return new_list

    elif hasattr(obj, 'todict'):
        return dict(obj.todict())

    else:
        try:
            json.dumps(obj)
        except TypeError:
            try:
                json.dumps(obj.__dict__)
            except TypeError:
                return str(obj)
            else:
                return obj.__dict__
        else:
            return obj